"""file: acyclic.py -- correction td11, exercices about finding cycle
in directed grphs

"""

def are_all_marked(li):
  """li is a list of boolean
  
  Returns the conjonction of all elements of li. In particular, it
  returns True on an empty list.

  """
  b = True
  for x in li: b = (b and x)
  return b

def succ(v):
  """graph is an adjacency matrix, v a vertex

  Returns the list of succesors of v in graph.

  """
  s = []
  for i in range(len(v)):
    if v[i]==1: s.append(i)
  return s

def find_vertex_all_succ_marked(graph,mark):
  """graph is an adjacency matrix, mark is a list of boolean

  Returns the first vertex such that all of its successors are marked,
  if it exists. Returns -1 otherwise.

  """
  for i in range(len(graph)):
    if (not mark[i]) and are_all_marked([mark[j] for j in succ(graph[i])]):
      return i
  return -1

def acyclic(graph):
  """graph is an adjacency matrix

  Returns True if and only if graph does not contain any cycle. It
  implements the following strategy: vertices can have two status,
  marked or unmarked; start with only unmarked vertices; while we can
  find a vertex whose sucessors are all marked (this is in particular
  the case if it has no successors!) then mark this vertex; at the
  end, if all vertices are marked return True, else return False.

  """
  n = len(graph) # number of vertices
  mark = [False for i in range(n)]
  v = find_vertex_all_succ_marked(graph,mark)
  while not (v==-1):
    mark[v] = True
    v = find_vertex_all_succ_marked(graph,mark)
  return are_all_marked(mark)


## testing area

## format: m is a matrix of 0 and 1. pp_matrix(m) prints out m in
## matrix form
def pp_matrix(m):
  print('\n'.join([''.join(['{:5,d}'.format(item) for item in row]) 
                   for row in m]))
  return None

# import random

# ntest=100
# for t in range(ntest):
#   graph = [[0 if i==j else random.randrange(0,2) for j in range(5)] for i in range(5)]
#   if acyclic(graph):
#     print "Test",t
#     pp_matrix(graph)
