"""file: votes.py -- correction td11, exercises about voting systems

"""

## format; path is a string, representing a file path. file2list(path)
## returns the list li, such that li[i] is the list of number appering
## on the ith line of the file at path.
def file2list (path):
  f,li = open(path,'r'),[]
  for line in f:
    li += [[int(k) for k in line.split()]]
  return li

# Careful! update modify m in place.
def update(vot,m):
  for i in range(len(vot)):
    for k in vot[i+1:]:
      m[vot[i]][k] += 1

# li must be non empty!
def duel(li):
  n = len(li[0])
  comp = [[0 for j in range(n)] for i in range(n)]
  for vot in li: update(vot,comp)
  return comp

def score(comp):
  li,n = [],len(comp)
  for i in range(n):
    for j in range(i+1,n):
      cij,cji = comp[i][j],comp[j][i]
      li += [[i,j,cij] if cij>cji else [j,i,cji]]
  return li


## Sorting function is heapsort(li,comp) in sort.py

import sort

## format: li is a list of triples. custom_sort(li) sorts li with the
## order relation comparing the last coordinate of the triples in
## reverse order.
def custom_sort(li): return sort.heapsort(li,lambda x,y: x[2]>=y[2])

## Functions for dertermining acyclicity are in acyclic.py

from acyclic import acyclic, pp_matrix

def tideman(votes):
  nvot,ncand = len(votes),len(votes[0])
  scores = custom_sort(score(duel(votes)))
  graph = [[0 for j in range(ncand)] for i in range(ncand)]
  for (i,j,_) in scores:
    graph[i][j] = 1
    if not acyclic(graph): graph[i][j] = 0
  return graph

def winner(graph):
  n = len(graph)
  for j in range(n):
    if [graph[i][j] for i in range(n)] == [0 for i in range(n)]:
      break
  return j

def find_source(li):
  for i in range(len(li)):
    if li[i] == 0: return i
  return None

def preflist(graph):
  n = len(graph)
  prefs = []
  # inedges[i] = number of edges entering i
  # value -1 means that i has been dismissed
  inedges = [sum([graph[i][j] for i in range(n)]) for j in range(n)]
  i = find_source(inedges)
  while i != None:
    prefs.append(i)
    for j in range(n): inedges[j] -= graph[i][j]
    inedges[i] = -1
    i = find_source(inedges)
  return prefs

## TEST ##

import random

## junk tests -- don't hesitate to toy with it
# for test in range(1000):
#   graph = [[random.randint(0,1) for j in range(6)] for i in range(6)]
#   for i in range(len(graph)): graph[i][i] = 0
#   if acyclic(graph):
#     print "test", test, "is acyclic:"
#     pp_matrix(graph)

# for (i,j,k) in custom_sort(score(duel(file2list("votesFile")))):
#   print i, "win over", j, "by", k, "voters"

# print "Winner is candidate", winner(tideman(file2list("votesFile")))

print "Candidates in preference order:",
for x in preflist(tideman(file2list("votesFile"))): print x,
print ""
